@@ -1,10 +1,15 @@
+from .constants import defines
 from .protocol import *
 from .commands import *
 from .parameters import *
 from .messages import *
 from .plugin_messages import *
 
+
 try:
+    from ixcom_internal.constants import *
+    from ixcom_internal.protocol import *
+    from ixcom_internal.commands import *
     from ixcom_internal.parameters import *
     from ixcom_internal.messages import *
     from ixcom_internal.plugin_messages import *
